@@ -8,18 +8,18 @@ online at https://average-minimum-distance.readthedocs.io.
 List of modules
 ***************
 
-===================    ===============================================================
+===================    ==============================================================
 Module                 Description
-===================    ===============================================================
-:mod:`.calculate`      Calculate invariants (fingerprints) of periodic sets (crystals)
+===================    ==============================================================
+:mod:`.calculate`      Calculate invariants (descriptors) of periodic sets (crystals)
 :mod:`.compare`        Compare fingerprints of crystals
 :mod:`.io`             Read periodic sets from a file or the CSD
 :mod:`.periodicset`    Implements the PeriodicSet object
-:mod:`.utils`          General utility functions/classes
-===================    ===============================================================
+:mod:`.utils`          General utility functions
+===================    ==============================================================
 """
 
-__version__ = '1.3.1'
+__version__ = '1.3.2'
 __author__ = 'Daniel Widdowson'
 __maintainer__ = 'Daniel Widdowson'
 __email__ = 'D.E.Widdowson@liverpool.ac.uk'
